#include <pthread.h>
#include <stdlib.h>

#include "ki/arc.h"
#include "ki/memory.h"
#include "ki/sys.h"

ARC* ARCNew(void *ptr) {
  ARC *arc = ki_malloc(sizeof(ARC));

  ARCInit(arc, ptr);

  return arc;
}

void ARCInit(ARC *arc, void *ptr) {
  arc->ptr = ptr;
  arc->refcount = 1;
  arc->lock = PTHREAD_MUTEX_INITIALIZER;
}

void ARCIncrement(ARC *arc) {
  (void)pthread_mutex_lock(&arc->lock);
  if (!arc->refcount) {
    fatal_error("ARC refcount is 0");
  }

  arc->refcount++;
  (void)pthread_mutex_unlock(&arc->lock);
}

void ARCDecrement(ARC *arc) {
  (void)pthread_mutex_lock(&arc->lock);
  if (!arc->refcount) {
    fatal_error("ARC refcount is 0");
  }

  arc->refcount--;

  if (!arc->refcount) {
    free(arc->ptr);
    arc->ptr = NULL;
  }
  (void)pthread_mutex_unlock(&arc->lock);
}

/* vi: set et ts=2 sw=2: */

